@@ -1,7 +1,7 @@
 # ----------------------------------------------------------------------------
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2019 pyglet contributors
+# Copyright (c) 2008-2021 pyglet contributors
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -32,8 +32,6 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 # ----------------------------------------------------------------------------
-from __future__ import print_function
-from __future__ import absolute_import
 
 from ctypes import *
 from .base import FontException
@@ -43,6 +41,7 @@ _libfreetype = pyglet.lib.load_library('freetype')
 
 _font_data = {}
 
+
 def _get_function(name, argtypes, rtype):
     try:
         func = getattr(_libfreetype, name)
@@ -50,7 +49,7 @@ def _get_function(name, argtypes, rtype):
         func.restype = rtype
         return func
     except AttributeError as e:
-            raise ImportError(e)
+        raise ImportError(e)
 
 
 FT_Byte = c_char
