@@ -1,13 +1,45 @@
-from builtins import object
-#!/usr/bin/python
-# $Id:$
+# ----------------------------------------------------------------------------
+# pyglet
+# Copyright (c) 2006-2008 Alex Holkner
+# Copyright (c) 2008-2021 pyglet contributors
+# All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions
+# are met:
+#
+#  * Redistributions of source code must retain the above copyright
+#    notice, this list of conditions and the following disclaimer.
+#  * Redistributions in binary form must reproduce the above copyright
+#    notice, this list of conditions and the following disclaimer in
+#    the documentation and/or other materials provided with the
+#    distribution.
+#  * Neither the name of pyglet nor the names of its
+#    contributors may be used to endorse or promote products
+#    derived from this software without specific prior written
+#    permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
+# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+# POSSIBILITY OF SUCH DAMAGE.
+# ----------------------------------------------------------------------------
 
 from pyglet import gl, compat_platform
 from pyglet.gl import gl_info
 from pyglet.gl import glu_info
 
-class Config(object):
-    '''Graphics configuration.
+
+class Config:
+    """Graphics configuration.
 
     A Config stores the preferences for OpenGL attributes such as the
     number of auxilliary buffers, size of the colour and depth buffers,
@@ -54,7 +86,7 @@ class Config(object):
         `accum_alpha_size` : int
             Bits per pixel devoted to the alpha component in the accumulation
             buffer.
-    '''
+    """
 
     _attribute_names = [
         'double_buffer',
@@ -85,20 +117,20 @@ class Config(object):
     debug = None
 
     def __init__(self, **kwargs):
-        '''Create a template config with the given attributes.
+        """Create a template config with the given attributes.
 
         Specify attributes as keyword arguments, for example::
 
             template = Config(double_buffer=True)
 
-        '''
+        """
         for name in self._attribute_names:
             if name in kwargs:
                 setattr(self, name, kwargs[name])
             else:
                 setattr(self, name, None)
 
-    def _requires_gl_3(self):
+    def requires_gl_3(self):
         if self.major_version is not None and self.major_version >= 3:
             return True
         if self.forward_compatible or self.debug:
@@ -106,16 +138,16 @@ class Config(object):
         return False
 
     def get_gl_attributes(self):
-        '''Return a list of attributes set on this config.
+        """Return a list of attributes set on this config.
 
         :rtype: list of tuple ``(name, value)``
         :return: All attributes, with unset attributes having a value of
             ``None``.
-        '''
+        """
         return [(name, getattr(self, name)) for name in self._attribute_names]
 
     def match(self, canvas):
-        '''Return a list of matching complete configs for the given canvas.
+        """Return a list of matching complete configs for the given canvas.
 
         .. versionadded:: 1.2
 
@@ -124,11 +156,11 @@ class Config(object):
                 Display to host contexts created from the config.
 
         :rtype: list of `CanvasConfig`
-        '''
+        """
         raise NotImplementedError('abstract')
 
     def create_context(self, share):
-        '''Create a GL context that satisifies this configuration.
+        """Create a GL context that satisifies this configuration.
 
         :deprecated: Use `CanvasConfig.create_context`.
 
@@ -138,13 +170,12 @@ class Config(object):
 
         :rtype: `Context`
         :return: The new context.
-        '''
-        raise gl.ConfigException(
-            'This config cannot be used to create contexts.  '
-            'Use Config.match to created a CanvasConfig')
+        """
+        raise gl.ConfigException('This config cannot be used to create contexts.  '
+                                 'Use Config.match to created a CanvasConfig')
 
     def is_complete(self):
-        '''Determine if this config is complete and able to create a context.
+        """Determine if this config is complete and able to create a context.
 
         Configs created directly are not complete, they can only serve
         as templates for retrieving a supported config from the system.
@@ -155,16 +186,16 @@ class Config(object):
 
         :rtype: bool
         :return: True if the config is complete and can create a context.
-        '''
+        """
         return isinstance(self, CanvasConfig)
 
     def __repr__(self):
         import pprint
-        return '%s(%s)' % (self.__class__.__name__, 
-                           pprint.pformat(self.get_gl_attributes()))
+        return '%s(%s)' % (self.__class__.__name__, pprint.pformat(self.get_gl_attributes()))
+
 
 class CanvasConfig(Config):
-    '''OpenGL configuration for a particular canvas.
+    """OpenGL configuration for a particular canvas.
 
     Use `Config.match` to obtain an instance of this class.
 
@@ -174,7 +205,8 @@ class CanvasConfig(Config):
         `canvas` : `Canvas`
             The canvas this config is valid on.
 
-    '''
+    """
+
     def __init__(self, canvas, base_config):
         self.canvas = canvas
 
@@ -187,7 +219,7 @@ class CanvasConfig(Config):
         raise NotImplementedError('abstract')
 
     def create_context(self, share):
-        '''Create a GL context that satisifies this configuration.
+        """Create a GL context that satisifies this configuration.
 
         :Parameters:
             `share` : `Context`
@@ -195,22 +227,23 @@ class CanvasConfig(Config):
 
         :rtype: `Context`
         :return: The new context.
-        '''
+        """
         raise NotImplementedError('abstract')
 
     def is_complete(self):
         return True
- 
 
-class ObjectSpace(object):
+
+class ObjectSpace:
     def __init__(self):
-        # Textures and buffers scheduled for deletion the next time this
-        # object space is active.
+        # Textures and buffers scheduled for deletion
+        # the next time this object space is active.
         self._doomed_textures = []
         self._doomed_buffers = []
 
-class Context(object):
-    '''OpenGL context for drawing.
+
+class Context:
+    """OpenGL context for drawing.
 
     Use `CanvasConfig.create_context` to create a context.
 
@@ -219,7 +252,7 @@ class Context(object):
             An object which is shared between all contexts that share
             GL objects.
 
-    '''
+    """
 
     #: Context share behaviour indicating that objects should not be
     #: shared with existing contexts.
@@ -228,7 +261,7 @@ class Context(object):
     #: Context share behaviour indicating that objects are shared with
     #: the most recently created context (the default).
     CONTEXT_SHARE_EXISTING = 1
-    
+
     # Used for error checking, True if currently within a glBegin/End block.
     # Ignored if error checking is disabled.
     _gl_begin = False
@@ -255,14 +288,14 @@ class Context(object):
         ('_workaround_vbo',
          lambda info: (info.get_renderer().startswith('ATI Radeon X')
                        or info.get_renderer().startswith('RADEON XPRESS 200M')
-                       or info.get_renderer() == 
-                            'Intel 965/963 Graphics Media Accelerator')),
+                       or info.get_renderer() ==
+                       'Intel 965/963 Graphics Media Accelerator')),
 
         # Some ATI cards on OS X start drawing from a VBO before it's written
         # to.  In these cases pyglet needs to call glFinish() to flush the
         # pipeline after updating a buffer but before rendering.
         ('_workaround_vbo_finish',
-         lambda info: ('ATI' in info.get_renderer() and 
+         lambda info: ('ATI' in info.get_renderer() and
                        info.have_version(1, 5) and
                        compat_platform == 'darwin')),
     ]
@@ -276,7 +309,7 @@ class Context(object):
             self.object_space = context_share.object_space
         else:
             self.object_space = ObjectSpace()
-    
+
     def __repr__(self):
         return '%s()' % self.__class__.__name__
 
@@ -324,13 +357,13 @@ class Context(object):
             self.object_space._doomed_buffers[0:len(buffers)] = []
 
     def destroy(self):
-        '''Release the context.
+        """Release the context.
 
         The context will not be useable after being destroyed.  Each platform
         has its own convention for releasing the context and the buffer(s)
         that depend on it in the correct order; this should never be called
         by an application.
-        '''
+        """
         self.detach()
 
         if gl.current_context is self:
@@ -342,7 +375,7 @@ class Context(object):
                 gl._shadow_window.switch_to()
 
     def delete_texture(self, texture_id):
-        '''Safely delete a texture belonging to this context.
+        """Safely delete a texture belonging to this context.
 
         Usually, the texture is released immediately using
         ``glDeleteTextures``, however if another context that does not share
@@ -353,7 +386,7 @@ class Context(object):
             `texture_id` : int
                 The OpenGL name of the texture to delete.
 
-        '''
+        """
         if self.object_space is gl.current_context.object_space:
             id = gl.GLuint(texture_id)
             gl.glDeleteTextures(1, id)
@@ -361,7 +394,7 @@ class Context(object):
             self.object_space._doomed_textures.append(texture_id)
 
     def delete_buffer(self, buffer_id):
-        '''Safely delete a buffer object belonging to this context.
+        """Safely delete a buffer object belonging to this context.
 
         This method behaves similarly to :py:func:`~pyglet.text.document.AbstractDocument.delete_texture`, though for
         ``glDeleteBuffers`` instead of ``glDeleteTextures``.
@@ -371,7 +404,7 @@ class Context(object):
                 The OpenGL name of the buffer to delete.
 
         .. versionadded:: 1.1
-        '''
+        """
         if self.object_space is gl.current_context.object_space and False:
             id = gl.GLuint(buffer_id)
             gl.glDeleteBuffers(1, id)
@@ -379,10 +412,10 @@ class Context(object):
             self.object_space._doomed_buffers.append(buffer_id)
 
     def get_info(self):
-        '''Get the OpenGL information for this context.
+        """Get the OpenGL information for this context.
 
         .. versionadded:: 1.2
 
         :rtype: `GLInfo`
-        '''
+        """
         return self._info
