@@ -1,7 +1,7 @@
 # ----------------------------------------------------------------------------
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2020 pyglet contributors
+# Copyright (c) 2008-2021 pyglet contributors
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -67,6 +67,8 @@ def load_material_library(filename):
     opacity = 1.0
     texture_name = None
 
+    matlib = {}
+
     for line in file:
         if line.startswith('#'):
             continue
@@ -75,7 +77,13 @@ def load_material_library(filename):
             continue
 
         if values[0] == 'newmtl':
+            if name is not None:
+                # save previous material
+                for item in (diffuse, ambient, specular, emission):
+                    item.append(opacity)
+                matlib[name] = Material(name, diffuse, ambient, specular, emission, shininess, texture_name)
             name = values[1]
+
         elif name is None:
             raise ModelDecodeException('Expected "newmtl" in '.format(filename))
 
@@ -103,7 +111,9 @@ def load_material_library(filename):
     for item in (diffuse, ambient, specular, emission):
         item.append(opacity)
 
-    return Material(name, diffuse, ambient, specular, emission, shininess, texture_name)
+    matlib[name] = Material(name, diffuse, ambient, specular, emission, shininess, texture_name)
+
+    return matlib
 
 
 def parse_obj_file(filename, file=None):
@@ -156,8 +166,7 @@ def parse_obj_file(filename, file=None):
 
         elif values[0] == 'mtllib':
             material_abspath = os.path.join(location, values[1])
-            material = load_material_library(filename=material_abspath)
-            materials[material.name] = material
+            materials = load_material_library(filename=material_abspath)            
 
         elif values[0] in ('usemtl', 'usemat'):
             material = materials.get(values[1])
