@@ -1,7 +1,7 @@
 # ----------------------------------------------------------------------------
 # pyglet
 # Copyright (c) 2006-2008 Alex Holkner
-# Copyright (c) 2008-2020 pyglet contributors
+# Copyright (c) 2008-2021 pyglet contributors
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -76,7 +76,7 @@ A simple example of drawing shapes::
 import math
 
 from pyglet.gl import GL_COLOR_BUFFER_BIT, GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA
-from pyglet.gl import GL_TRIANGLES, GL_TRIANGLE_STRIP, GL_LINES, GL_BLEND
+from pyglet.gl import GL_TRIANGLES, GL_LINES, GL_BLEND
 from pyglet.gl import glPushAttrib, glPopAttrib, glBlendFunc, glEnable, glDisable
 from pyglet.graphics import Group, Batch
 
@@ -308,45 +308,103 @@ class _ShapeBase:
 
 
 class Arc(_ShapeBase):
-    def __init__(self, x, y, radius, segments=25, angle=math.pi * 2, color=(255, 255, 255), batch=None, group=None):
-        # TODO: Finish this shape and add docstring.
+    def __init__(self, x, y, radius, segments=None, angle=math.tau, start_angle=0,
+                 closed=False, color=(255, 255, 255), batch=None, group=None):
+        """Create an Arc.
+
+        The Arc's anchor point (x, y) defaults to it's center.
+
+        :Parameters:
+            `x` : float
+                X coordinate of the circle.
+            `y` : float
+                Y coordinate of the circle.
+            `radius` : float
+                The desired radius.
+            `segments` : int
+                You can optionally specifify how many distict line segments
+                the arc should be made from. If not specified it will be
+                automatically calculated using the formula:
+                `max(14, int(radius / 1.25))`.
+            `angle` : float
+                The angle of the arc, in radians. Defaults to tau (pi * 2),
+                which is a full circle.
+            `start_angle` : float
+                The start angle of the arc, in radians. Defaults to 0.
+            `closed` : bool
+                If True, the ends of the arc will be connected with a line.
+                defaults to False.
+            `color` : (int, int, int)
+                The RGB color of the circle, specified as a tuple of
+                three ints in the range of 0-255.
+            `batch` : `~pyglet.graphics.Batch`
+                Optional batch to add the circle to.
+            `group` : `~pyglet.graphics.Group`
+                Optional parent group of the circle.
+        """
         self._x = x
         self._y = y
         self._radius = radius
-        self._segments = segments
+        self._segments = segments or max(14, int(radius / 1.25))
+        self._num_verts = self._segments * 2 + (2 if closed else 0)
+
         self._rgb = color
         self._angle = angle
+        self._start_angle = start_angle
+        self._closed = closed
+        self._rotation = 0
 
         self._batch = batch or Batch()
         self._group = _ShapeGroup(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, group)
 
-        self._vertex_list = self._batch.add(self._segments * 2, GL_LINES, self._group, 'v2f', 'c4B')
+        self._vertex_list = self._batch.add(self._num_verts, GL_LINES, self._group, 'v2f', 'c4B')
         self._update_position()
         self._update_color()
 
     def _update_position(self):
         if not self._visible:
-            vertices = (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
+            vertices = (0,) * self._segments * 4
         else:
             x = self._x + self._anchor_x
             y = self._y + self._anchor_y
             r = self._radius
-            tau_segs = self._angle / (self._segments - 1)
+            tau_segs = self._angle / self._segments
+            start_angle = self._start_angle - math.radians(self._rotation)
 
             # Calcuate the outer points of the arc:
-            points = [(x + (r * math.cos(i * tau_segs)),
-                       y + (r * math.sin(i * tau_segs))) for i in range(self._segments)]
+            points = [(x + (r * math.cos((i * tau_segs) + start_angle)),
+                       y + (r * math.sin((i * tau_segs) + start_angle))) for i in range(self._segments + 1)]
 
             # Create a list of doubled-up points from the points:
             vertices = []
-            for i, point in enumerate(points):
-                line_points = *points[i - 1], *point
+            for i in range(len(points) - 1):
+                line_points = *points[i], *points[i + 1]
                 vertices.extend(line_points)
 
+            if self._closed:
+                chord_points = *points[-1], *points[0]
+                vertices.extend(chord_points)
+
         self._vertex_list.vertices[:] = vertices
 
     def _update_color(self):
-        self._vertex_list.colors[:] = [*self._rgb, int(self._opacity)] * self._segments * 2
+        self._vertex_list.colors[:] = [*self._rgb, int(self._opacity)] * self._num_verts
+
+    @property
+    def rotation(self):
+        """Clockwise rotation of the arc, in degrees.
+
+        The arc will be rotated about its (anchor_x, anchor_y)
+        position.
+
+        :type: float
+        """
+        return self._rotation
+
+    @rotation.setter
+    def rotation(self, rotation):
+        self._rotation = rotation
+        self._update_position()
 
     def draw(self):
         """Draw the shape at its current position.
@@ -372,8 +430,9 @@ class Circle(_ShapeBase):
                 The desired radius.
             `segments` : int
                 You can optionally specifify how many distict triangles
-                the circle should be made from. If not specified, it will
-                be automatically calculated based on the radius.
+                the circle should be made from. If not specified it will
+                be automatically calculated based using the formula:
+                `max(14, int(radius / 1.25))`.
             `color` : (int, int, int)
                 The RGB color of the circle, specified as a tuple of
                 three ints in the range of 0-255.
@@ -385,7 +444,7 @@ class Circle(_ShapeBase):
         self._x = x
         self._y = y
         self._radius = radius
-        self._segments = segments or int(radius / 1.25)
+        self._segments = segments or max(14, int(radius / 1.25))
         self._rgb = color
 
         self._batch = batch or Batch()
